-- ********************************************************************************
-- Broker Statistics
-- A Data Broker display for combat statistics.
-- By: Shenton
--
-- Config.lua
-- Ace3 configuration panel and databases.
-- ********************************************************************************

local A = _G["DataBrokerStatisticsGlobal"];

-- Lua globals
local format, pairs, smatch, strlen = format, pairs, string.match, string.len;

-- ********************************************************************************
-- Configuration
-- ********************************************************************************

--- Default database table
local db = {};
function A:AceDefaultDB()
    db =
    {
        profile =
        {
            verbosity = 1,
            time = 1,
            displayPre = 1,
            displayPost = 1,
            shareWhisper = 1,
            shareGuild = 1,
            shareRaid = 1,
            shareBg = 1,
            statsSeparator = "|",
            configAccordingSpecBehavior = 1,
            statIconSize = 16,
            preColor =
            {
                r = 0,
                g = 0.5,
                b = 0.5,
                a = 1
            },
            preColorHex = "|cff008080",
            postColor =
            {
                r = 1,
                g = 1,
                b = 1,
                a = 1
            },
            postColorHex = "|cffffffff",
            separatorColor =
            {
                r = 1,
                g = 1,
                b = 1,
                a = 1
            },
            separatorColorHex = "|cffffffff",
            builtInStats = {},
            custom = {},
            dataBroker =
            {
                defaultObjectIcon = A.defaultLdbIcon;
                customDisplays = {}
            }
        },
        global =
        {
            custom = {},
            customDisplays = {}
        }
    };

    return db;
end

--- AceConfig menu
local options, addName, deleteCustom, deleteDbObject, addString = {}, nil, {}, {}, nil;
function A:AceConfig()
    options =
    {
        name = A.L["Broker Statistics"],
        type = "group",
        childGroups = "tab",
        args =
        {
            saveMethod =
            {
                order = 0,
                name = A.L["Save configuration method: %s"]:format(A.playerSpec),
                type = "description"
            },
            options =
            {
                order = 10,
                name = A.L["Options"],
                type = "group",
                args =
                {
                    main =
                    {
                        order = 0,
                        name = A.L["Main"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            verbosity =
                            {
                                order = 0,
                                name = A.L["Verbosity"],
                                desc = A.L["Set the addon chat messages verbosity."],
                                type = "select",
                                values = { [0] = A.L["Disabled"], [1] = A.L["Normal"], [2] = A.L["Everything"] },
                                set = function(info, val) A.db.profile.verbosity = val; end,
                                get = function(info) return A.db.profile.verbosity; end
                            },
                            refreshRate =
                            {
                                order = 1,
                                name = A.L["Refresh Rate"],
                                desc = A.L["The refresh rate in seconds."],
                                type = "range",
                                min = 0.1,
                                max = 5,
                                step = 0.1,
                                set = function(info, val)
                                    A.db.profile.time = val;
                                    A:RefreshTimer();
                                end,
                                get = function(info) return A.db.profile.time; end
                            }
                        }
                    },
                    display =
                    {
                        order = 1,
                        name = A.L["Display"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            pre =
                            {
                                order = 0,
                                name = A.L["Pre"],
                                desc = A.L["Display the stat prefix."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.displayPre = val; end,
                                get = function(info) return A.db.profile.displayPre; end
                            },
                            post =
                            {
                                order = 1,
                                name = A.L["Post"],
                                desc = A.L["Display the stat post string."],
                                type = "toggle",
                                set = function(info, val) A.db.profile.displayPost = val; end,
                                get = function(info) return A.db.profile.displayPost; end
                            },
                            separator =
                            {
                                order = 2,
                                name = A.L["Separator"],
                                desc = A.L["Enter the separator displayed between each stat."],
                                type = "input",
                                set = function(info, val)
                                    A.db.profile.statsSeparator = val;
                                end,
                                get = function() return A.db.profile.statsSeparator; end
                            },
                            lineBreak1 =
                            {
                                order = 10,
                                name = "",
                                type = "description",
                                width = "full"
                            },
                            preColor =
                            {
                                order = 11,
                                name = A.L["Pre Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.preColor.r = r;
                                    A.db.profile.preColor.g = g;
                                    A.db.profile.preColor.b = b;
                                    A.db.profile.preColor.a = a;
                                    A.db.profile.preColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.preColor.r,
                                        A.db.profile.preColor.g,
                                        A.db.profile.preColor.b,
                                        A.db.profile.preColor.a
                                end
                            },
                            postColor =
                            {
                                order = 12,
                                name = A.L["Post Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.postColor.r = r;
                                    A.db.profile.postColor.g = g;
                                    A.db.profile.postColor.b = b;
                                    A.db.profile.postColor.a = a;
                                    A.db.profile.postColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.postColor.r,
                                        A.db.profile.postColor.g,
                                        A.db.profile.postColor.b,
                                        A.db.profile.postColor.a
                                end
                            },
                            separatorColor =
                            {
                                order = 13,
                                name = A.L["Separator Color"],
                                desc = A.L["Pickup a color."],
                                type = "color",
                                hasAlpha = true,
                                set = function(info, r, g, b, a)
                                    A.db.profile.separatorColor.r = r;
                                    A.db.profile.separatorColor.g = g;
                                    A.db.profile.separatorColor.b = b;
                                    A.db.profile.separatorColor.a = a;
                                    A.db.profile.separatorColorHex = "|c"..A:PercentToHex(r, g, b, a);
                                end,
                                get = function()
                                    return A.db.profile.separatorColor.r,
                                        A.db.profile.separatorColor.g,
                                        A.db.profile.separatorColor.b,
                                        A.db.profile.separatorColor.a
                                end
                            }
                        }
                    },
                    configurationBehavior =
                    {
                        order = 3,
                        name = A.L["Configuration save behavior"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            accordingToSpec =
                            {
                                order = 0,
                                name = A.L["According to specialisation"],
                                desc = A.L["Enable if the configuration will be saved according to the current specialisation."],
                                type = "toggle",
                                set = function(info, val)
                                    A.db.profile.configAccordingSpec = val;
                                    A:RefreshConfig();
                                end,
                                get = function(info) return A.db.profile.configAccordingSpec; end
                            },
                            specSaveBehavior =
                            {
                                order = 1,
                                name = A.L["Specialisation save behavior"],
                                desc = A.L["Choose if the configuration will be saved just on the specialisation name or the specialisation name and numbers (xx/xx/xx).\n\nBe aware that when selecting a mode for the first time you will got an empty configuration."],
                                type = "select",
                                values = function() return { [1] = A.L["Specialisation name"], [2] = A.L["Specialisation name and numbers"] } end,
                                disabled = function() return not A.db.profile.configAccordingSpec; end,
                                set = function(info, val)
                                    A.db.profile.configAccordingSpecBehavior = val;
                                    A:RefreshConfig();
                                end,
                                get = function(info) return A.db.profile.configAccordingSpecBehavior; end
                            },
                        }
                    },
                    shareReject =
                    {
                        order = 4,
                        name = A.L["Reject share request"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            whisper =
                            {
                                order = 0,
                                name = A.L["Player"],
                                desc = A.L["Reject share from other players."],
                                type = "toggle",
                                get = function() return A.db.profile.shareWhisper; end,
                                set = function(info, val) A.db.profile.shareWhisper = val; end
                            },
                            guild =
                            {
                                order = 1,
                                name = A.L["Guild"],
                                desc = A.L["Reject share from guild."],
                                type = "toggle",
                                get = function() return A.db.profile.shareGuild; end,
                                set = function(info, val) A.db.profile.shareGuild = val; end
                            },
                            raid =
                            {
                                order = 2,
                                name = A.L["Raid"],
                                desc = A.L["Reject share from raid or party."],
                                type = "toggle",
                                get = function() return A.db.profile.shareRaid; end,
                                set = function(info, val) A.db.profile.shareRaid = val; end
                            },
                            bg =
                            {
                                order = 3,
                                name = A.L["Battleground"],
                                desc = A.L["Reject share from battleground."],
                                type = "toggle",
                                get = function() return A.db.profile.shareBg; end,
                                set = function(info, val) A.db.profile.shareBg = val; end
                            }
                        }
                    }
                }
            },
            dataBroker =
            {
                order = 20,
                name = A.L["Data Broker"],
                type = "group",
                args =
                {
                    addNew =
                    {
                        order = 0,
                        name = A.L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            inputAdd =
                            {
                                order = 0,
                                name = A.L["Enter Name"],
                                desc = A.L["Enter the new Data Broker object name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end
                            },
                            executeAdd =
                            {
                                order = 1,
                                name = A.L["Create"],
                                disabled = function() return not addName; end,
                                desc = A.L["Create the new Data Broker object %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(A.L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.customDisplays[addName] ) then
                                        A:Message(A.L["The Data Broker object %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.customDisplays[addName] = 1;
                                        A.db.profile.dataBroker.customDisplays[A.playerSpec][addName] =
                                        {
                                            enabled = 1
                                        };
                                        addName = nil;
                                        A:RefreshConfig();
                                    end
                                end
                            }
                        }
                    }
                }
            },
            builtInStats =
            {
                order = 30,
                name = A.L["Built in Stats"],
                type = "group",
                childGroups = "tab",
                args =
                {
                    misc =
                    {
                        order = 0,
                        name = A.L["Miscellaneous"],
                        type = "group",
                        args = {}
                    },
                    tank =
                    {
                        order = 1,
                        name = A.L["Tank"],
                        type = "group",
                        args = {}
                    },
                    melee =
                    {
                        order = 2,
                        name = A.L["Melee"],
                        type = "group",
                        args = {}
                    },
                    ranged =
                    {
                        order = 3,
                        name = A.L["Ranged"],
                        type = "group",
                        args = {}
                    },
                    spell =
                    {
                        order = 4,
                        name = A.L["Spell"],
                        type = "group",
                        args = {}
                    }
                }
            },
            customStats =
            {
                order = 40,
                name = A.L["Custom Stats"],
                type = "group",
                args =
                {
                    addNew =
                    {
                        order = 0,
                        name = A.L["Add New"],
                        type = "group",
                        inline = true,
                        args =
                        {
                            header1 =
                            {
                                order = 0,
                                name = A.L["Create New"],
                                type = "header"
                            },
                            inputAdd =
                            {
                                order = 1,
                                name = A.L["Enter Name"],
                                desc = A.L["Enter the new statistic name."],
                                type = "input",
                                set = function(info, val) addName = val; end,
                                get = function() return addName; end
                            },
                            executeAdd =
                            {
                                order = 2,
                                name = A.L["Create"],
                                disabled = function() return not addName; end,
                                desc = A.L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    addName = strtrim(addName);
                                    if ( not addName or addName == "" ) then
                                        A:Message(A.L["The name provided was empty."], 1, 1);
                                        return;
                                    end

                                    if ( A.db.global.custom[addName] ) then
                                        A:Message(A.L["The custom stat %s already exists."]:format(addName), 1, 1);
                                        addName = nil;
                                    elseif ( A.builtInStats[addName] ) then
                                        A:Message(A.L["A built in stat with that name already exists."], 1, 1);
                                        addName = nil;
                                    else
                                        A.db.global.custom[addName] = {};
                                        addName = nil;
                                    end
                                end
                            },
                            header10 =
                            {
                                order = 10,
                                name = A.L["Import from string"],
                                type = "header"
                            },
                            inputAddString =
                            {
                                order = 11,
                                name = A.L["Paste String"],
                                desc = A.L["Paste here the new statistic string."],
                                type = "input",
                                multiline = 1,
                                width = "full",
                                set = function(info, val) addString = val; end,
                                get = function() return addString; end
                            },
                            executeAddString =
                            {
                                order = 12,
                                name = A.L["Create"],
                                disabled = function() return not addString; end,
                                desc = A.L["Create the new custom statistic %s"]:format(addName or ""),
                                type = "execute",
                                func = function(info, val)
                                    local customName = smatch(addString, "^(.+){");
                                    local customText = smatch(addString, "{(.+)}");
                                    customText = "{"..customText.."}";

                                    if ( A.db.global.custom[customName] ) then
                                        local pop = StaticPopup_Show("BrokerStatsAddCustomNameExists");
                                        if (pop) then pop.customText = customText; end
                                        addString = nil;
                                    elseif ( A.builtInStats[customName] ) then
                                        local pop = StaticPopup_Show("BrokerStatsAddCustomNameExists");
                                        if (pop) then pop.customText = customText; end
                                        addString = nil;
                                    else
                                        local customTable = loadstring("return "..customText);
                                        customTable = customTable();

                                        if ( customName ) then
                                            A.db.global.custom[customName] = customTable;
                                            A.AceConfigRegistry:NotifyChange("BrokerStatsConfig");
                                            addString = nil;
                                        else
                                            A:Message(A.L["The name provided was empty."], 1, 1);
                                        end
                                    end
                                end
                            }
                        }
                    }
                }
            }
        }
    };

    if ( A:GotCustomLdb() ) then
        options.args.dataBroker.args.list =
        {
            order = 1,
            name = A.L["List"],
            type = "group",
            inline = true,
            args = {}
        };
    end

    local order = 0;
    for k,v in A:PairsByKeys(A.db.global.customDisplays) do
        options.args.dataBroker.args.list.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enabled =
                {
                    order = 0,
                    name = A.L["Enable"],
                    desc = A.L["Enable this Data Broker object."],
                    type = "toggle",
                    set = function(info, val)
                        A.db.profile.dataBroker.customDisplays[A.playerSpec][k].enabled = val;

                        if ( not val ) then
                            local pop = StaticPopup_Show("BrokerStatsDisablingLdbObject");
                        end

                        A:RefreshConfig();
                    end,
                    get = function(info)
                        if ( not A.db.profile.dataBroker.customDisplays[A.playerSpec][k] ) then
                            A.db.profile.dataBroker.customDisplays[A.playerSpec][k] = {};
                        end

                        return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].enabled;
                    end
                },
                selectIcon =
                {
                    order = 11,
                    name = function()
                        return A.L["Select Icon"]
                    end,
                    desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    type = "execute",
                    image = function(info)
                        if ( A.db.profile.dataBroker.customDisplays[A.playerSpec][k].icon ) then
                            return A.db.profile.dataBroker.customDisplays[A.playerSpec][k].icon, 36, 36;
                        else
                            return A.defaultLdbIcon, 36, 36;
                        end
                    end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.dataBroker.customDisplays[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.dataBroker.customDisplays[A.playerSpec][k];
                            BrokerStatisticsPopupFrame.ldbObject = A["ldbObject"..k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end
                },
                lineBreak2 =
                {
                    order = 20,
                    name = "",
                    type = "description",
                    width = "full"
                },
                deleteToggle =
                {
                    order = 21,
                    name = A.L["Delete"],
                    desc = A.L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteDbObject[k] = val; end,
                    get = function(info) return deleteDbObject[k]; end
                },
                deleteExecute =
                {
                    order = 22,
                    name = A.L["Delete"],
                    desc = A.L["Delete the %s Data Broker object.\n\nThis will require reloading the user interface.\n\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteDbObject[k]; end,
                    type = "execute",
                    func = function() 
                        A.db.global.customDisplays[k] = nil;
                        ReloadUI();
                    end
                }
            }
        };
        order = order + 1;
    end

    order = 0;
    local dbObjectsSelect = A:CreateDbObjectsSelect();
    for k,v in A:PairsByKeys(A.builtInStats) do
        options.args.builtInStats.args[v.type].args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                enable =
                {
                    order = 0,
                    name = A.L["Enable"],
                    desc = A.L["Enable %s on the data broker display.\n\nCalculation:\n|cff4e5780%s"]:format(k, v.calc),
                    type = "toggle",
                    set = function(info, val)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then
                            A.db.profile.builtInStats[A.playerSpec][k] =
                            {
                                enabled = val
                            };
                        else
                            A.db.profile.builtInStats[A.playerSpec][k].enabled = val;
                        end
                    end,
                    get = function(info)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end
                },
                enableIcon =
                {
                    order = 1,
                    name = A.L["Enable Icon"],
                    desc = A.L["Enable the icon to be displayed before the statistic"],
                    type = "toggle",
                    disabled = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].iconEnabled = val; end,
                    get = function(info)
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.builtInStats[A.playerSpec][k].iconEnabled;
                    end
                },
                selectIcon =
                {
                    order = 2,
                    name = function()
                        return A.L["Select Icon"]
                    end,
                    desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    type = "execute",
                    disabled = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].iconEnabled;
                    end,
                    image = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].icon ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].icon, 36, 36;
                        else
                            return A.defaultStatIcon, 36, 36;
                        end
                    end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.builtInStats[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.builtInStats[A.playerSpec][k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end
                },
                lineBreak1 =
                {
                    order = 20,
                    name = "",
                    type = "description",
                    width = "full"
                },
                color =
                {
                    order = 21,
                    name = A.L["Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    disabled = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.builtInStats[A.playerSpec][k].r = r;
                        A.db.profile.builtInStats[A.playerSpec][k].g = g;
                        A.db.profile.builtInStats[A.playerSpec][k].b = b;
                        A.db.profile.builtInStats[A.playerSpec][k].a = a;
                        A.db.profile.builtInStats[A.playerSpec][k].colorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        if ( A.db.profile.builtInStats[A.playerSpec][k] ) then
                            if ( A.db.profile.builtInStats[A.playerSpec][k].r ) then
                                return A.db.profile.builtInStats[A.playerSpec][k].r,
                                    A.db.profile.builtInStats[A.playerSpec][k].g,
                                    A.db.profile.builtInStats[A.playerSpec][k].b,
                                    A.db.profile.builtInStats[A.playerSpec][k].a;
                            else
                                A.db.profile.builtInStats[A.playerSpec][k].r = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].g = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].b = 1;
                                A.db.profile.builtInStats[A.playerSpec][k].a = 1;
                            end
                        end

                        return 1, 1 , 1, 1;
                    end
                },
                dataBrokerObject =
                {
                    order = 22,
                    name = A.L["Data Broker Object"],
                    desc = A.L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    disabled = function()
                        if ( not A.db.profile.builtInStats[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.builtInStats[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.builtInStats[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.builtInStats[A.playerSpec][k] and A.db.profile.builtInStats[A.playerSpec][k].dbObject ) then
                            return A.db.profile.builtInStats[A.playerSpec][k].dbObject;
                        else
                            return A.L["Default"];
                        end
                    end
                },
            }
        };
        order = order + 1;
    end

    order = 1;
    for k,v in A:PairsByKeys(A.db.global.custom) do
        local calc = v.calc or A.L["Not Available"];

        options.args.customStats.args[k] =
        {
            order = order,
            name = k,
            type = "group",
            inline = true,
            args =
            {
                header1 =
                {
                    order = 0,
                    name = A.L["Main"],
                    type = "header"
                },
                enabled =
                {
                    order = 1,
                    name = A.L["Enable"],
                    desc = A.L["Enable %s on the data broker display.\n\nCalculation:\n|cff4e5780%s"]:format(k, calc),
                    type = "toggle",
                    set = function(info, val)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then
                            A.db.profile.custom[A.playerSpec][k] =
                            {
                                enabled = val
                            };
                        else
                            A.db.profile.custom[A.playerSpec][k].enabled = val;
                        end
                    end,
                    get = function(info)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.custom[A.playerSpec][k].enabled;
                    end
                },
                enableIcon =
                {
                    order = 2,
                    name = A.L["Enable Icon"],
                    desc = A.L["Enable the icon to be displayed before the statistic"],
                    type = "toggle",
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].iconEnabled = val; end,
                    get = function(info)
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return nil; end

                        return A.db.profile.custom[A.playerSpec][k].iconEnabled;
                    end
                },
                selectIcon =
                {
                    order = 3,
                    name = function()
                        return A.L["Select Icon"]
                    end,
                    desc = A.L["Select the Data Broker object main icon.\n\n|cFFC79C6ELeft-Click:|r Default icon list (Macro frame).\n|cFFC79C6EShift+Left-Click:|r Item macro list (Guild bank frame)."],
                    type = "execute",
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].iconEnabled;
                    end,
                    image = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].icon ) then
                            return A.db.profile.custom[A.playerSpec][k].icon, 36, 36;
                        else
                            return A.defaultStatIcon, 36, 36;
                        end
                    end,
                    func = function()
                        if ( not BrokerStatisticsPopupFrame:IsVisible() ) then
                            BrokerStatisticsPopupFrame:ClearAllPoints();
                            BrokerStatisticsPopupFrame:SetPoint("CENTER", GetMouseFocus(), "CENTER", 150, 0);

                            if ( IsShiftKeyDown() ) then BrokerStatisticsPopupFrame.iconList = 1; end
                            BrokerStatisticsPopupFrame.currentTexture = A.db.profile.custom[A.playerSpec][k].icon;
                            BrokerStatisticsPopupFrame.dbEdit = A.db.profile.custom[A.playerSpec][k];

                            BrokerStatisticsPopupFrame:Show();
                            A:PopupScrollUpdate();
                        end
                    end
                },
                lineBreak10 =
                {
                    order = 10,
                    name = "",
                    type = "description",
                    width = "full"
                },
                color =
                {
                    order = 11,
                    name = A.L["Color"],
                    desc = A.L["Pickup a color."],
                    type = "color",
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].enabled;
                    end,
                    hasAlpha = true,
                    set = function(info, r, g, b, a)
                        A.db.profile.custom[A.playerSpec][k].r = r;
                        A.db.profile.custom[A.playerSpec][k].g = g;
                        A.db.profile.custom[A.playerSpec][k].b = b;
                        A.db.profile.custom[A.playerSpec][k].a = a;
                        A.db.profile.custom[A.playerSpec][k].colorHex = "|c"..A:PercentToHex(r, g, b, a);
                    end,
                    get = function()
                        if ( A.db.profile.custom[A.playerSpec][k] ) then
                            if ( A.db.profile.custom[A.playerSpec][k].r ) then
                                return A.db.profile.custom[A.playerSpec][k].r,
                                    A.db.profile.custom[A.playerSpec][k].g,
                                    A.db.profile.custom[A.playerSpec][k].b,
                                    A.db.profile.custom[A.playerSpec][k].a;
                            else
                                A.db.profile.custom[A.playerSpec][k].r = 1;
                                A.db.profile.custom[A.playerSpec][k].g = 1;
                                A.db.profile.custom[A.playerSpec][k].b = 1;
                                A.db.profile.custom[A.playerSpec][k].a = 1;
                            end
                        end

                        return 1, 1 , 1, 1;
                    end
                },
                dataBrokerObject =
                {
                    order = 12,
                    name = A.L["Data Broker Object"],
                    desc = A.L["Choose on which Data Broker object to display %s."]:format(k),
                    type = "select",
                    values = dbObjectsSelect,
                    disabled = function()
                        if ( not A.db.profile.custom[A.playerSpec][k] ) then return 1; end

                        return not A.db.profile.custom[A.playerSpec][k].enabled;
                    end,
                    set = function(info, val) A.db.profile.custom[A.playerSpec][k].dbObject = val; end,
                    get = function(info)
                        if ( A.db.profile.custom[A.playerSpec][k] and A.db.profile.custom[A.playerSpec][k].dbObject ) then
                            return A.db.profile.custom[A.playerSpec][k].dbObject;
                        else
                            return A.L["Default"];
                        end
                    end
                },
                lineBreak20 =
                {
                    order = 20,
                    name = "",
                    type = "description",
                    width = "full"
                },
                deleteToggle =
                {
                    order = 21,
                    name = A.L["Delete"],
                    desc = A.L["Activate the delete button."],
                    type = "toggle",
                    set = function(info, val) deleteCustom[k] = val; end,
                    get = function(info) return deleteCustom[k]; end
                },
                deleteExecute =
                {
                    order = 22,
                    name = A.L["Delete"],
                    desc = A.L["Delete the %s custom statistic.\n|cffff3333No more warning is given beyond this point."]:format(k),
                    disabled = function() return not deleteCustom[k]; end,
                    type = "execute",
                    func = function()
                        A.db.global.custom[k] = nil;
                        A.db.profile.custom["primary"][k] = nil;
                        A.db.profile.custom["secondary"][k] = nil;
                    end
                },
                header30 =
                {
                    order = 30,
                    name = A.L["Options"],
                    type = "header"
                },
                pre =
                {
                    order = 31,
                    name = A.L["Pre"],
                    desc = A.L["The text to display before the statistic, ex: \"AP: \"."],
                    type = "input",
                    set = function(info, val) A.db.global.custom[k].pre = val; end,
                    get = function(info, val) return A.db.global.custom[k].pre; end
                },
                post =
                {
                    order = 32,
                    name = A.L["Post"],
                    desc = A.L["The text to display after the statistic, ex: \"%\"\nNote: this is displayed after any stat in multiple mode."],
                    type = "input",
                    set = function(info, val) A.db.global.custom[k].post = val; end,
                    get = function(info, val) return A.db.global.custom[k].post; end
                },
                lineBreak40 =
                {
                    order = 40,
                    name = "",
                    type = "description",
                    width = "full"
                },
                multiple =
                {
                    order = 41,
                    name = A.L["Multiple"],
                    desc = A.L["Tell the display engine there is more than one statistic returned."],
                    type = "toggle",
                    disabled = function() return not A.db.global.custom[k].separator; end,
                    set = function(info, val) A.db.global.custom[k].multiple = val; end,
                    get = function(info) return A.db.global.custom[k].multiple; end
                },
                separator =
                {
                    order = 42,
                    name = A.L["Separator"],
                    desc = A.L["The separator used in multiple mode."],
                    type = "input",
                    set = function(info, val)
                        if ( val == "" ) then
                            val = nil;
                            A.db.global.custom[k].multiple = nil;
                        end
                        A.db.global.custom[k].separator = val; end,
                    get = function(info, val) return A.db.global.custom[k].separator; end
                },
                lineBreak50 =
                {
                    order = 50,
                    name = "",
                    type = "description",
                    width = "full"
                },
                round =
                {
                    order = 51,
                    name = A.L["Round"],
                    desc = A.L["Round the result to the specified decimal places."],
                    type = "select",
                    values = { [1000] = A.L["Disabled"], [0] = 0, [1] = 1, [2] = 2, [3] = 3, [4] = 4, [5] = 5 },
                    set = function(info, val)
                        if ( val == 1000 ) then
                            A.db.global.custom[k].round = nil;
                        else
                            A.db.global.custom[k].round = val;
                        end
                    end,
                    get = function(info)
                        if ( A.db.global.custom[k].round ) then
                            return A.db.global.custom[k].round;
                        else
                            return 1000;
                        end
                    end
                },
                calc =
                {
                    order = 52,
                    name = A.L["Calculation Code"],
                    desc = A.L["The code used to calculate the statistic."],
                    type = "input",
                    multiline = 1,
                    width = "full",
                    set = function(info, val) A.db.global.custom[k].calc = val; end,
                    get = function(info, val) return A.db.global.custom[k].calc; end
                },
                header60 =
                {
                    order = 60,
                    name = A.L["Share"],
                    type = "header"
                },
                sharePlayer =
                {
                    order = 61,
                    name = A.L["Player"],
                    desc = A.L["Send %s to another player."]:format(k),
                    type = "execute",
                    func = function()
                        local pop = StaticPopup_Show("BrokerStatsSendCustom");
                        if (pop) then pop.customStat = k; end
                    end
                },
                shareGuild =
                {
                    order = 62,
                    name = A.L["Guild"],
                    desc = A.L["Send %s to your guild."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("GUILD", k); end
                },
                shareRaid =
                {
                    order = 63,
                    name = A.L["Raid"],
                    desc = A.L["Send %s to your raid. It will send it to your group if you are in a party."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("RAID", k); end
                },
                shareBg =
                {
                    order = 64,
                    name = A.L["Battleground"],
                    desc = A.L["Send %s to your battleground raid."]:format(k),
                    type = "execute",
                    func = function() A:SendStat("BATTLEGROUND", k); end
                },
                shareInput =
                {
                    order = 65,
                    name = A.L["String"],
                    desc = A.L["Use this to store this custom stat."],
                    type = "input",
                    multiline = 1,
                    width = "full",
                    get = function() return k..A:TableTostring(v); end,
                    set = false
                }
            }
        };
        order = order + 1;
    end

    -- Profiles
    options.args.profile = A.AceDBOptions:GetOptionsTable(A.db);
    options.args.profile.order = 1000;

    A:CreateStatisticsTable();

    return options;
end
